(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[1242],{29221:function(e,r,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/privacy-requests/configure",function(){return t(48566)}])},43124:function(e,r,t){"use strict";t.d(r,{Z:function(){return p}});var i=t(24246),s=t(5702),n=t(88038),o=t.n(n),a=t(86677);t(27378);var d=t(11596),c=t(72247),l=t(11032),u=()=>{let e=(0,a.useRouter)();return(0,i.jsx)(s.xuv,{bg:"gray.50",border:"1px solid",borderColor:"blue.400",borderRadius:"md",justifyContent:"space-between",p:5,mb:5,mt:5,children:(0,i.jsxs)(s.xuv,{children:[(0,i.jsxs)(s.Kqy,{direction:{base:"column",sm:"row"},justifyContent:"space-between",children:[(0,i.jsx)(s.xvT,{fontWeight:"semibold",children:"Configure your storage and messaging provider"}),(0,i.jsx)(s.wpx,{onClick:()=>{e.push(l.fz)},children:"Configure"})]}),(0,i.jsxs)(s.xvT,{children:["Before Fides can process your privacy requests we need two simple steps to configure your storage and email client."," "]})]})})},p=e=>{let{children:r,title:t,padded:n=!0,mainProps:l}=e,p=(0,d.hz)(),g=(0,a.useRouter)(),h="/privacy-requests"===g.pathname||"/datastore-connection"===g.pathname,f=!(p.flags.privacyRequestsConfiguration&&h),{data:m}=(0,c.JE)(void 0,{skip:f}),{data:x}=(0,c.PW)(void 0,{skip:f}),y=p.flags.privacyRequestsConfiguration&&(!m||!x)&&h;return(0,i.jsxs)(s.kCb,{"data-testid":t,direction:"column",h:"100vh",children:[(0,i.jsxs)(o(),{children:[(0,i.jsxs)("title",{children:["Fides Admin UI - ",t]}),(0,i.jsx)("meta",{name:"description",content:"Privacy Engineering Platform"}),(0,i.jsx)("link",{rel:"icon",href:"/favicon.ico"})]}),(0,i.jsxs)(s.kCb,{as:"main",direction:"column",py:n?6:0,px:n?10:0,h:n?"calc(100% - 48px)":"full",flex:1,minWidth:0,overflow:"auto",...l,children:[y?(0,i.jsx)(u,{}):null,r]})]})}},37541:function(e,r,t){"use strict";var i=t(24246),s=t(5702),n=t(18622);r.Z=e=>{let{heading:r,breadcrumbItems:t,isSticky:o=!0,children:a,rightContent:d,style:c,...l}=e;return(0,i.jsxs)("div",{...l,style:o?{position:"sticky",top:"-24px",paddingTop:"24px",paddingBottom:"24px",paddingLeft:"40px",marginLeft:"-40px",paddingRight:"40px",marginRight:"-40px",marginTop:"-24px",left:0,zIndex:20,backgroundColor:"white",...c}:{paddingBottom:"24px",...c},children:[(0,i.jsxs)(s.jqI,{justify:"space-between",children:["string"==typeof r?(0,i.jsx)(s.X6q,{className:t||a?"pb-4":void 0,fontSize:"2xl","data-testid":"page-heading",children:r}):r,d&&(0,i.jsx)("div",{"data-testid":"page-header-right-content",children:d})]}),!!t&&(0,i.jsx)(n.m,{className:a?"pb-4":void 0,items:t,"data-testid":"page-breadcrumb"}),a]})}},18622:function(e,r,t){"use strict";t.d(r,{m:function(){return c}});var i=t(24246),s=t(5702),n=t(79894),o=t.n(n),a=t(27378);let{Text:d}=s.AntTypography,c=e=>{let{items:r,...t}=e,n=(0,a.useMemo)(()=>null==r?void 0:r.map((e,t)=>{let s=t===r.length-1,n={...e};return"string"==typeof n.title&&(n.title=(0,i.jsx)(d,{style:{color:"inherit",maxWidth:s?void 0:400},ellipsis:!s,children:n.title})),n.icon&&(n.title=(0,i.jsxs)(i.Fragment,{children:[(0,i.jsx)("span",{className:"anticon align-text-bottom",children:n.icon}),n.title]})),n.href&&n.title&&(n.title=(0,i.jsx)(o(),{href:n.href,className:"ant-breadcrumb-link",children:n.title}),delete n.href),n}),[r]);return(0,i.jsx)(s.zrq,{items:n,...t})}},48566:function(e,r,t){"use strict";t.r(r);var i=t(24246),s=t(5702),n=t(79894),o=t.n(n),a=t(43124),d=t(11032),c=t(37541);let{Text:l}=s.AntTypography;r.default=()=>(0,i.jsxs)(a.Z,{title:"Configure Privacy Requests",children:[(0,i.jsx)(c.Z,{heading:"Privacy Requests",breadcrumbItems:[{title:"All requests",href:d.Jw},{title:"Configure requests"}]}),(0,i.jsx)(l,{strong:!0,children:"Configure your privacy requests:"}),(0,i.jsxs)(s.xuv,{display:"flex",alignItems:"center",my:5,"data-testid":"privacy-requests-configure",children:[(0,i.jsxs)(s.fGe,{p:"5",borderWidth:"1px",rounded:"md",borderColor:"gray.300",_hover:{borderColor:"complimentary.500",cursor:"pointer"},mr:5,minHeight:"100%",children:[(0,i.jsx)(s.X6q,{mb:2,size:"sm",children:(0,i.jsx)(s.AB5,{as:o(),href:"/privacy-requests/configure/messaging",children:"Configure messaging provider"})}),"Fides supports email (Mailgun & Twilio) and SMS (Twilio) server configurations for sending processing notices to privacy request subjects. Configure your settings here."]}),(0,i.jsxs)(s.fGe,{p:"5",borderWidth:"1px",rounded:"md",borderColor:"gray.300",_hover:{borderColor:"complimentary.500",cursor:"pointer"},minHeight:"100%",children:[(0,i.jsx)(s.X6q,{mb:2,size:"sm",children:(0,i.jsx)(s.AB5,{as:o(),href:"/privacy-requests/configure/storage",children:"Configure storage"})}),"The data produced by an access request will need to be uploaded to a storage destination (e.g. an S3 bucket) in order to be returned to the user. At least one storage destination must be configured to process access requests. Configure your settings here."]})]})]})}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=29221)}),_N_E=e.O()}]);